
%%
 
%public
%class FreeFormLexerPhase1
%throws Exception
%char
%line
%column
%ignorecase
%type Token
%{
	// First phase of free form lexical analysis for Fortran 95 parser
	// (see FreeFormLexerPhase2.java and f95t.bnf)
	// Jeff Overbey

	private Token token(int id)
	{
		Token t = new Token();
		t.id = id;
		t.filename = "<stdin>";
		t.offset = yychar;
		t.length = yylength();
		t.text = (id == Token.T_SCON ? stringBuffer.toString() : yytext());
		t.startLine = yyline+1; t.startCol = yycolumn+1;

		// Iterate through the string to determine the ending line and column
		t.endLine = t.startLine; t.endCol = t.startCol-1;
		for (int i = 0; i < t.length; i++)
		{
			if (t.text.charAt(i) == '\n')
			{
				t.endLine++;
				t.endCol = 1;
			}
			else t.endCol++;
		}

		return t;
	}

	public static void main(String[] args) throws Exception
	{
		FreeFormLexerPhase2 l = new FreeFormLexerPhase2(System.in);
		Parser.parse(l);
	}

	private StringBuffer stringBuffer = null;

	private boolean wantEos = false;
%}

LineTerminator = \r\n|\n|\r
WhiteSpace     = [ \t\f]+

Comment = "!" [^\r\n]*

StarredType = ("*"[0-9]+)?

Dig = [0-9]+
EExp = ("E"|"e")("+"|"-")?{Dig}
DExp = ("D"|"d")("+"|"-")?{Dig}
Sig1 = ({Dig} "." [0-9]+ | "." {Dig})
Sig2 = {Dig} "."
Sig = {Sig1} | {Sig2}
DdotD = {Dig} "." {Dig}
Efw = [Ee] {Dig}
Range = [a-zA-Z](-[a-zA-Z])?

Bcon = ( B ' [0-1]+ ' | B \" [0-1]+ \" )
Ocon = ( O ' [0-7]+ ' | O \" [0-7]+ \" )
Zcon = ( Z ' [0-9A-Fa-f]+ ' | Z \" [0-9A-Fa-f]+ \" )

Dcon1 = {Dig}{DExp} | {Sig}{DExp} | {Sig1}
Dcon2 = {Sig2}
Fcon = [IiBbOoZzFfDd] {WhiteSpace}* {DdotD} | (E [NnSs]? | e [NnSs]? | G | g) {WhiteSpace}* {DdotD} {Efw}?
Icon = ( {Dig} | {Dig} ("E"|"e") {Dig} "." {Dig} )
Pcon = ("+"|"-")? {Dig} (P|p)
Rcon1 = {Dig}{EExp} | {Sig}{EExp} | {Sig1}
Rcon2 = {Sig2}
Xcon = {Dig} (X|x)
Xdop = \. [A-Za-z]+ \.
Ident = [A-Za-z][A-Za-z0-9_]*
xImpl = "(" {Range} ("," {Range})* ")"
xImplLkahead = [ \t\f]* (","|";"|{LineTerminator})

NumDotLkahead = [^A-Za-z]

LineContinuation = "&"{WhiteSpace}*{Comment}?{LineTerminator}{WhiteSpace}*"&"?

%state IMPLICIT
%state QUOTED
%state DBLQUOTED

%%

/* Lexical rules */

<YYINITIAL,IMPLICIT> {
"ACCESS"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ACCESSEQ); }
"ACTION"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ACTIONEQ); }
"ADVANCE"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ADVANCEEQ); }
"ALLOCATABLE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ALLOCATABLE); }
"ALLOCATE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ALLOCATE); }
".AND."				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_AND); }
"ASSIGNMENT"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ASSIGNMENT); }
"*"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ASTERISK); }
"BACKSPACE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_BACKSPACE); }
"BLANK"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_BLANKEQ); }
"BLOCK"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_BLOCK); }
"BLOCKDATA"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_BLOCKDATA); }
"CALL"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_CALL); }
"CASE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_CASE); }
"CHARACTER"{StarredType}	{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_CHARACTER); }
"CLOSE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_CLOSE); }
":"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_COLON); }
","				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_COMMA); }
"COMMON"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_COMMON); }
"COMPLEX"{StarredType}		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_COMPLEX); }
"CONTAINS"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_CONTAINS); }
"CONTINUE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_CONTINUE); }
"CYCLE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_CYCLE); }
"DATA"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_DATA); }
"DEALLOCATE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_DEALLOCATE); }
"DEFAULT"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_DEFAULT); }
"DELIM"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_DELIMEQ); }
"DIMENSION"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_DIMENSION); }
"DIRECT"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_DIRECTEQ); }
"DO"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_DO); }
"DOUBLE"{StarredType}		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_DOUBLE); }
"DOUBLEPRECISION"{StarredType}	{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_DOUBLEPRECISION); }
"ELEMENTAL"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ELEMENTAL); }
"ELSE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ELSE); }
"ELSEIF"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ELSEIF); }
"ELSEWHERE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ELSEWHERE); }
"END"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_END); }
"ENDBLOCK"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDBLOCK); }
"ENDBLOCKDATA"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDBLOCKDATA); }
"ENDDO"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDDO); }
"END"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDEQ); }
"ENDFILE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDFILE); }
"ENDFORALL"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDFORALL); }
"ENDFUNCTION"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDFUNCTION); }
"ENDIF"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDIF); }
"ENDINTERFACE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDINTERFACE); }
"ENDMODULE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDMODULE); }
"ENDPROGRAM"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDPROGRAM); }
"ENDSELECT"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDSELECT); }
"ENDSUBROUTINE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDSUBROUTINE); }
"ENDTYPE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDTYPE); }
"ENDWHERE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENDWHERE); }
"ENTRY"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ENTRY); }
".EQ."				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_EQ); }
"=="				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_EQEQ); }
"=>"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_EQGREATERTHAN); }
"="				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_EQUALS); }
"EQUIVALENCE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_EQUIVALENCE); }
".EQV."				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_EQV); }
"ERR"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ERREQ); }
"EXIST"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_EXISTEQ); }
"EXIT"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_EXIT); }
"EXTERNAL"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_EXTERNAL); }
".FALSE."			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_FALSE); }
"FILE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_FILE); }
"FILE"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_FILEEQ); }
"FMT"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_FMTEQ); }
"FORALL"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_FORALL); }
"FORMAT"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_FORMAT); }
"FORMATTED"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_FORMATTEDEQ); }
"FORM"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_FORMEQ); }
"FUNCTION"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_FUNCTION); }
".GE."				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_GE); }
"GO"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_GO); }
"GOTO"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_GOTO); }
">"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_GREATERTHAN); }
">="				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_GREATERTHANEQ); }
".GT."				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_GT); }
"IF"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_IF); }
"IMPLICIT"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_IMPLICIT); }
"IN"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_IN); }
"INCLUDE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_INCLUDE); }
"INOUT"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_INOUT); }
"INQUIRE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_INQUIRE); }
"INTEGER"{StarredType}		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_INTEGER); }
"INTENT"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_INTENT); }
"INTERFACE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_INTERFACE); }
"INTRINSIC"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_INTRINSIC); }
"IOLENGTH"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_IOLENGTHEQ); }
"IOSTAT"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_IOSTATEQ); }
"KIND"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_KINDEQ); }
".LE."				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_LE); }
"LEN"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_LENEQ); }
"<"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_LESSTHAN); }
"<="				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_LESSTHANEQ); }
"LOGICAL"{StarredType}		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_LOGICAL); }
"("				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_LPAREN); }
"(/"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_LPARENSLASH); }
".LT."				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_LT); }
"-"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_MINUS); }
"MODULE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_MODULE); }
"NAMED"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_NAMEDEQ); }
"NAME"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_NAMEEQ); }
"NAMELIST"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_NAMELIST); }
".NE."				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_NE); }
".NEQV."			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_NEQV); }
"NEXTREC"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_NEXTRECEQ); }
"NML"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_NMLEQ); }
"NONE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_NONE); }
".NOT."				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_NOT); }
"NULL"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_NULL); }
"NULLIFY"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_NULLIFY); }
"NUMBER"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_NUMBEREQ); }
"ONLY"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ONLY); }
"OPEN"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_OPEN); }
"OPENED"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_OPENEDEQ); }
"OPERATOR"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_OPERATOR); }
"OPTIONAL"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_OPTIONAL); }
".OR."				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_OR); }
"OUT"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_OUT); }
"PAD"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PADEQ); }
"PARAMETER"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PARAMETER); }
"PAUSE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PAUSE); }
"%"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PERCENT); }
"+"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PLUS); }
"POINTER"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_POINTER); }
"POSITION"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_POSITIONEQ); }
"**"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_POW); }
"PRECISION"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PRECISION); }
"PRINT"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PRINT); }
"PRIVATE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PRIVATE); }
"PROCEDURE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PROCEDURE); }
"PROGRAM"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PROGRAM); }
"PUBLIC"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PUBLIC); }
"PURE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PURE); }
"READ"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_READ); }
"READ"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_READEQ); }
"READWRITE"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_READWRITEEQ); }
"REAL"{StarredType}		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_REAL); }
"REC"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_RECEQ); }
"RECL"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_RECLEQ); }
"RECURSIVE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_RECURSIVE); }
"RESULT"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_RESULT); }
"RETURN"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_RETURN); }
"REWIND"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_REWIND); }
")"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_RPAREN); }
"SAVE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_SAVE); }
"SELECT"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_SELECT); }
"SELECTCASE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_SELECTCASE); }
"SEQUENCE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_SEQUENCE); }
"SEQUENTIAL"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_SEQUENTIALEQ); }
"SIZE"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_SIZEEQ); }
"/"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_SLASH); }
"/="				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_SLASHEQ); }
"/)"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_SLASHRPAREN); }
"//"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_SLASHSLASH); }
"STAT"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_STATEQ); }
"STATUS"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_STATUSEQ); }
"STOP"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_STOP); }
"SUBROUTINE"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_SUBROUTINE); }
"TARGET"			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_TARGET); }
"THEN"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_THEN); }
"TO"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_TO); }
".TRUE."			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_TRUE); }
"TYPE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_TYPE); }
"_"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_UNDERSCORE); }
"UNFORMATTED"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_UNFORMATTEDEQ); }
"UNIT"[ \t]*"="			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_UNITEQ); }
"USE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_USE); }
"WHERE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_WHERE); }
"WHILE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_WHILE); }
"WRITE"				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_WRITE); }
"WRITE"[ \t]*"="		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_WRITEEQ); }
{Bcon}				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_BCON); }
{Dcon1}				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_DCON); }
{Dcon2}/{NumDotLkahead}		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_DCON); }
{Fcon}				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_FCON); }
{Icon}				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ICON); }
{Ident}				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_IDENT); }
{Ocon}				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_OCON); }
{Pcon}				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_PCON); }
{Rcon1}				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_RCON); }
{Rcon2}/{NumDotLkahead}		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_RCON); }
{Xcon}				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_XCON); }
{Xdop}				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_XDOP); }
{Zcon}				{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_ZCON); }
"'"				{ stringBuffer = new StringBuffer();
				  stringBuffer.append('\'');
				  yybegin(QUOTED); }
\"				{ stringBuffer = new StringBuffer();
				  stringBuffer.append('\"');
				  yybegin(DBLQUOTED); }
{Comment}			{ }
{WhiteSpace}			{ }
{LineContinuation}		{ }
{LineTerminator}|";"		{ boolean b = wantEos; wantEos = false; if (b) return token(Token.T_EOS); }
<<EOF>>				{ wantEos = false; yybegin(YYINITIAL); return token(Token.END_OF_INPUT); }
}

<IMPLICIT> {
{xImpl}/{xImplLkahead}		{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_xImpl); }
{xImpl}$			{ wantEos = true; yybegin(YYINITIAL); return token(Token.T_xImpl); }
}

<QUOTED> {
"''"				{ stringBuffer.append('\''); }
"'"				{ stringBuffer.append('\'');
				  yybegin(YYINITIAL);
				  wantEos = true;
				  return token(Token.T_SCON); }
[^\n\r&']+			{ stringBuffer.append( yytext() ); }
{LineContinuation}		{ }
"&"				{ stringBuffer.append( yytext() ); }
[\n\r]				{ throw new Exception("Lexer Error (line " + (yyline+1) + ", col " + (yycolumn+1) + "): String literal spans multiple lines without continuation"); }
<<EOF>>				{ throw new Exception("Lexer Error (line " + (yyline+1) + ", col " + (yycolumn+1) + "): End of file encountered before string literal terminated"); }
}

<DBLQUOTED> {
\"\"				{ stringBuffer.append('\"'); }
\"				{ stringBuffer.append('\"');
				  yybegin(YYINITIAL);
				  wantEos = true;
				  return token(Token.T_SCON); }
[^\n\r&\"]+			{ stringBuffer.append( yytext() ); }
{LineContinuation}		{ }
"&"				{ stringBuffer.append( yytext() ); }
[\n\r]				{ throw new Exception("Lexer Error (line " + (yyline+1) + ", col " + (yycolumn+1) + "): String literal spans multiple lines without continuation"); }
<<EOF>>				{ throw new Exception("Lexer Error (line " + (yyline+1) + ", col " + (yycolumn+1) + "): End of file encountered before string literal terminated"); }
}

/* Error */
.				{ throw new Exception("Lexer Error (line " + (yyline+1) + ", col " + (yycolumn+1) + "): Unexpected character " + yytext()); }

